#!/bin/bash
if [ -z "$(pgrep pulseaudio)" ] && [ -z "$(pgrep pipewire-pulse)" ]; then
  zenity --warning --title="PulseAudioの設定" --text="PulseAudioが動作していません" --no-wrap
  exit
fi
if [ ! -f ~/.config/pulse/daemon.conf ]; then
  mkdir -p ~/.config/pulse
  cp /etc/konahqss/daemon.conf ~/.config/pulse
fi
if [ ! -f ~/.config/pulse/default.pa ]; then
  cp /etc/pulse/default.pa ~/.config/pulse
fi
samplerate="44100!48000!88200!96000!176400!192000!352800!384000"
sampleformat="u8!s16le!s16be!s24le!s24be!s32le!s32be!s24-32le!s24-32be!float32le!float32be!ulaw!alaw"
#accelvals="1!2!3!4!5!6!7!8!9!10"
sr0="$(grep ^default-sample-rate ~/.config/pulse/daemon.conf)"
sr1="$(grep ^alternate-sample-rate ~/.config/pulse/daemon.conf)"
sr="$(grep ^default-sample-rate ~/.config/pulse/daemon.conf|cut -d' ' -f3)"
sm0="$(grep ^default-sample-format ~/.config/pulse/daemon.conf)"
sm="$(grep ^default-sample-format ~/.config/pulse/daemon.conf|cut -d' ' -f3)"
resamplemethod="$(pulseaudio --dump-resample-methods)"
resamplemethod="$(echo $resamplemethod|sed -e 's/ /\!/g')"
rsm0="$(grep ^resample-method ~/.config/pulse/daemon.conf)"
rsm="$(grep ^resample-method ~/.config/pulse/daemon.conf|cut -d' ' -f3)"
#if [ "$(grep "^daemonize = yes" ~/.config/pulse/daemon.conf)" ]; then
#  daemon="TRUE"
#else
#  daemon="FALSE"
#fi
#if [ ! -z "$(grep "^load-module module-udev-detect" ~/.config/pulse/default.pa|grep tsched=0)" ]; then
#  accel0="TRUE"
#else
  accel0="FALSE"
#fi
#if [ ! -z "$(grep "high-priority = yes" ~/.config/pulse/daemon.conf)" ]; then
#  priority="TRUE"
#else
#  priority="FALSE"
#fi
if [ -x /usr/bin/qpaeq ]; then
  ret=$(yad --form --center --title="PulseAudioの設定" --text="ここで音源のサンプルレートに合わせます\nサンプルレートを変えるときとくにアップサンプリングのときは\n高いサンプルレートに応じて高精度な方式にする必要があります\n" --field="サンプルレート(Hz):CB" --field="サンプルフォーマット:CB" --field="リサンプル方式:CB" --image=/usr/share/pixmaps/sonata.png "$sr!$samplerate" "$sm!$sampleformat" "$rsm!$resamplemethod" --button="Equalizer(_E):2" --button="キャンセル(_C):1" --button="OK(_O):0")
else
  ret=$(yad --form --center --title="PulseAudioの設定" --text="ここで音源のサンプルレートに合わせます\nサンプルレートを変えるときとくにアップサンプリング(*)のときは\n高いサンプルレートに応じて高精度な方式にする必要があります\n(*)アップサンプリングは基本的に推奨しません\n" --field="サンプルレート(Hz):CB" --field="サンプルフォーマット:CB" --field="リサンプル方式:CB" --image=/usr/share/pixmaps/sonata.png "$sr!$samplerate" "$sm!$sampleformat" "$rsm!$resamplemethod" --button="キャンセル(_C):1" --button="OK(_O):0")
fi
rcd="$?"
if [ "$rcd" = "0" ]; then
  sr=$(echo $ret|cut -d'|' -f1)
  sm=$(echo $ret|cut -d'|' -f2)
  rsm=$(echo $ret|cut -d'|' -f3)
#  daemon2=$(echo $ret|cut -d'|' -f4)
  sed -i -e "s/$sr0/default-sample-rate = $sr/" ~/.config/pulse/daemon.conf
  sed -i -e "s/$sr1/alternate-sample-rate = $sr/" ~/.config/pulse/daemon.conf
  if [ ! -f ~/.config/pipewire/pipewire.conf ]; then
    mkdir -p ~/.config/pipewire
    cp /usr/share/pipewire/pipewire.conf ~/.config/pipewire
  fi
  sr00="$(grep default.clock.rate ~/.config/pipewire/pipewire.conf|awk '{print $3}')"
  sed -i -e "s/$sr00/$sr/" ~/.config/pipewire/pipewire.conf
  sed -i -e "s/$sm0/default-sample-format = $sm/" ~/.config/pulse/daemon.conf
  sed -i -e "s/$rsm0/resample-method = $rsm/" ~/.config/pulse/daemon.conf
#  if [ "$daemon2" != "$daemon" ]; then
#  if [ "$daemon2" = "TRUE" ]; then
#    sed -i -e "s/; daemonize = no/daemonize = yes/" ~/.config/pulse/daemon.conf
#  else
#    sed -i -e "s/daemonize = yes/; daemonize = no/" ~/.config/pulse/daemon.conf
#  fi
#  fi
#  tsched0="$(grep "^load-module module-udev-detect" ~/.config/pulse/default.pa)"
  if [ "$accel" = "TRUE" ] && [ "$accel0" = "FALSE" ]; then
    sed -i -e "s/$tsched0/load-module module-udev-detect tsched=0/" ~/.config/pulse/default.pa
    buffer_size="$(pacmd list sink|grep buffer_size|cut -f2 -d'"')"
    buffer_size="$(echo $buffer_size|cut -d' ' -f1)"
    fragment_size="$(pacmd list sink|grep fragment_size|cut -f2 -d'"')"
    fragment_size="$(echo $fragment_size|cut -d' ' -f1)"
    fragment="$(expr $buffer_size / $fragment_size)"
    sample_rate="$(grep ^default-sample-rate ~/.config/pulse/daemon.conf|cut -d' ' -f3)"
    sample_format="$(grep ^default-sample-format ~/.config/pulse/daemon.conf|cut -d' ' -f3)"
    ch="2"
    if [ "$sample_format" = "s16le" ]; then
      bit_depth="16"
    else
      bit_depth="32"
    fi
    bitrate="$(expr $sample_rate \* $bit_depth \* $ch)"
    fragment_size_ms="$(expr $fragment_size \* 1000 / $bitrate)"
    echo fragment_size_ms=$fragment_size_ms
    if [ "$fragment_size_ms" = "0" ]; then
      fragment_size_ms="1"
    fi
    if [ "$fragment_size_ms" -gt 100 ]; then
      fragment_size_ms="100"
    fi
    echo fragment_size_ms=$fragment_size_ms
    if [ ! -z "$fragment" ] && [ "$fragment" != "0" ]; then
      fragment0="$(grep default-fragments ~/.config/pulse/daemon.conf)"
      sed -i -e "s/$fragment0/default-fragments = $fragment/" ~/.config/pulse/daemon.conf
    fi
    if [ "$buffer_size" -gt 4096 ] && [ ! -z "$fragment_size_ms" ]; then
      fragment_size_ms0="$(grep default-fragment-size-msec ~/.config/pulse/daemon.conf)"
      sed -i -e "s/$fragment_size_ms0/default-fragment-size-msec = $fragment_size_ms/" ~/.config/pulse/daemon.conf
    fi
  fi
#  if [ "$priority" = "TRUE" ]; then
#    sed -i -e "s/high-priority = no/high-priority = yes/" ~/.config/pulse/daemon.conf
#  else
#    sed -i -e "s/high-priority = yes/high-priority = no/" ~/.config/pulse/daemon.conf
#  fi
  if [ ! -z "$(pgrep pulseaudio)" ]; then
    zenity --warning --text="PulseAudioを再起動するのでプレイヤーは再生を止めてください" --no-wrap
    pulseaudio --kill
    if [ ! -z "$(pgrep xfce4-volumed)" ]; then
      sleep 3
      killall xfce4-volumed
      xfce4-volumed
    fi
  fi
  zenity --warning --title="PulseAudioの設定" --text="PulseAudioを設定しました。（ただちに反映）" --no-wrap
elif [ "$rcd" = "2" ]; then
  if [ -x /usr/bin/qpaeq ]; then
    qpaeq
  fi
fi

